/*********************************************************
File:   Sunsensors.c
Author: Jan Rohde
Date:   4.6.07
Description:  functions for measuring the satellites
              position based on the suns position.
*********************************************************/

#include "sunsensors.h"

int sun_measurements[SUN_SENSORS];


void get_sun_measurements(float *sum_meas){
  /*
  starting TimerB in capture mode to measure the frequency of the LF converters.
  TimerB setup: Continous count
                Timer B Interrupt Vector, TBIV. Interrupt on rising edge.

  Used Registers: TBCTL
                  TBCCTL1-6
                  TBIV
  */
  int i, timer_b_overflowcounter, measurement_done;
  float CCR_values[6];
  float CCR_values_prev[6];
  float CCR_values_diff[6];
  float sun_measurements[3];

  int timer_b[6];
  int timer_b_done[6];
  int timer_b_overflow[6];

  //getting measurements
  TBCCR1 = 0;
  TBCCR2 = 0;
  TBCCR3 = 0;
  TBCCR4 = 0;
  TBCCR5 = 0;
  TBCCR6 = 0;
  timer_b_overflowcounter = 0;           //number of times timer_b goes around
  measurement_done = FALSE;
  
  TBCTL = TBSSEL_0 + MC_2 + TBCLR + TBIE;
                                          //remember to stop TimerB when reading register
  while(!(measurement_done)){             //input logic to set measurement_done = 1
    
  }
  
  //correct for timer B overflow. Done by adding 65536 * (number of overflows) to last value before subtraction
  for (i = 0; i < 6; i++){
    CCR_values[i] = CCR_values[i] + (65536f * (float)timer_b_overflow[i]);
    CCR_values_diff[i] = CCR_values[i] - CCR_values_prev[i];  //period lenght of each signal
  } 
  sun_measurements[0] = CCR_values_diff[0] - CCR_values_diff[5];
  sun_measurements[1] = CCR_values_diff[1] - CCR_values_diff[4];
  sun_measurements[2] = CCR_values_diff[2] - CCR_values_diff[3];

  sun_meas = &sun_measurements[0];                            //sun_meas now points to the first value of sun_measurements
}

#pragma vector = TIMERB1_VECTOR
__interrupt void Timer_B1(void){

  switch(TBIV){
    case 2:   if (timer_b[0] == 0){
                CCR_values_prev[0] = TBCCR1;
                timer_b[0] ++;
              }
              else if (timer_b[0] == 1){
                CCR_values[0] = TBCCR1;
                timer_b_done[0] = 1;
                timer_b_overflow[0] = timer_b_overflowcounter;
                timer_b[0] ++;
              }
              else if (timber_b[0] == 2)
                asm("nop");
              break;
              
    case 4:   if (timer_b[1] == 0){
                CCR_values_prev[1] = TBCCR1;
                timer_b[1] ++;
              }
              else if (timer_b[1] == 1){
                CCR_values[1] = TBCCR1;
                timer_b_done[1] = 1;
                timer_b_overflow[1] = timer_b_overflowcounter;
                timer_b[1] ++;
              }
              else if (timber_b[1] == 2)
                asm("nop");
              break;
              
    case 6:   if (timer_b[2] == 0){
                CCR_values_prev[2] = TBCCR1;
                timer_b[2] ++;
              }
              else if (timer_b[2] == 1){
                CCR_values[2] = TBCCR1;
                timer_b_done[2] = 1;
                timer_b_overflow[2] = timer_b_overflowcounter;
                timer_b[2] ++;
              }
              else if (timber_b[2] == 2)
                asm("nop");
              break;
              
    case 8:   if (timer_b[3] == 0){
                CCR_values_prev[3] = TBCCR1;
                timer_b[3] ++;
              }
              else if (timer_b[3] == 1){
                CCR_values[3] = TBCCR1;
                timer_b_done[3] = 1;
                timer_b_overflow[3] = timer_b_overflowcounter;
                timer_b[3] ++;
              }
              else if (timber_b[3] == 2)
                asm("nop");
              break;
              
    case 10:   if (timer_b[4] == 0){
                CCR_values_prev[4] = TBCCR1;
                timer_b[4] ++;
              }
              else if (timer_b[4] == 1){
                CCR_values[4] = TBCCR1;
                timer_b_done[4] = 1;
                timer_b_overflow[4] = timer_b_overflowcounter;
                timer_b[4] ++;
              }
              else if (timber_b[4] == 2)
                asm("nop");
              break;
              
    case 12:   if (timer_b[5] == 0){
                CCR_values_prev[5] = TBCCR1;
                timer_b[5] ++;
              }
              else if (timer_b[5] == 1){
                CCR_values[5] = TBCCR1;
                timer_b_done[5] = 1;
                timer_b_overflow[5] = timer_b_overflowcounter;
                timer_b[5] ++;
              }
              else if (timber_b[5] == 2)
                asm("nop");
              break;
              
    case 14:  timer_b_overflow++;
              break;
  }
}
