%Initialization file
%Author: Jan Rohde

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Global variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global omega_o;
global I;
global G;
global H;

load magfield10.mat;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Satellite model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
omega_o = -1.083e-3;                     %Angular velocity in orbit
colum = 2;
euler_0 = [20 30 40];             %Initial condition
w_B_OB = [0.005 0.003 0.000]';
q_0 = euler2q(pi/180*euler_0);
R_B_O = rquat(q_0);
c2_0 = R_B_O(:,2);
w_B_IB_0 = w_B_OB+omega_o*c2_0;
w_b_rb = [0 0 0]';

%inertia
Ix = 0.0061;
Iy = 0.0061;
Iz = 0.0028;
I = [Ix 0 0; 0 Iy 0; 0 0 Iz];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Orbit estimator
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global r_p;
M_earth = 5.9742e24;

h = 6.378e6+600.00e3;       %earth radius + orbit height
r_p = [h 0 0]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Noise
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

QuaternionNoise = 8.5e-9;
WhiteNoise = 8.5e-12;
MagneticNoise = 4.331e-6;
SunNoise = 0.0;

N_x = 100;
N_y = 100;
N_z = 100;
A_x = 0.0075; 
A_y = 0.0075;
A_z = 0.0075;
i_max = 20;

[G,H] = IGRF2000;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Controller
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

h_r = 0.0;
e_r = 0.0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Extended Kalman Filter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global Q H G R q_opt_next q_bar_next omega_bar_next x_bar_next P_bar_next

%Matrices
R = diag([3.3884e-6 3.3202e-7 3.8270e-7]);   %measurement covariance
Q = diag([8.5e-9 8.5e-9 8.5e-9 8.5e-12 8.5e-12 8.5e-12]);   %process covariance

G = eye(6);     %noise
H = [eye(3) zeros(3,3)];     %Measurement matrix

%init values
q_opt_next = [0.5 0.5 0.5 0.5]';    %gauss-newton
q_bar_next = [0.8289 -0.1881 0.4922 -0.1881]';   %attitude prediction
omega_bar_next = [0.001 0.001 0.001]';   %angular rate prediction
x_bar_next = [q_bar_next; omega_bar_next]';
P_bar_next = diag([1e-6 1e-6 1e-6 1e-6 1e-6 1e-6]); %error covariance

