function J = genJacobian(quatY_o)
% Generates Jacoban matrix for the Gauss-Newton
% Author: Jan Rohde
% input = [q_opt B_meas S_meas]'
% Returns: J

quat = quatY_o(1:4); %selects the quaternion from the input quatY_o
y_o = quatY_o(5:10); %selects the measurements from the input quatY_o

dR_deta = 2*[quat(1) -quat(4) quat(3);quat(4) quat(1) -quat(2); -quat(3) quat(2) quat(1)];
dR_deps1 = 2*[quat(2) quat(3) quat(4);quat(3) -quat(2) -quat(1); quat(4) quat(1) -quat(2)];
dR_deps2 = 2*[-quat(3) quat(2) quat(1);quat(2) quat(3) quat(4); -quat(1) quat(4) -quat(3)];
dR_deps3 = 2*[-quat(4) -quat(1) quat(2);quat(1) -quat(4) quat(3); quat(2) quat(3) quat(4)];

dM_deta = [dR_deta zeros(3,3); zeros(3,3) dR_deta];
dM_deps1 = [dR_deps1 zeros(3,3); zeros(3,3) dR_deps1];
dM_deps2 = [dR_deps2 zeros(3,3); zeros(3,3) dR_deps2];
dM_deps3 = [dR_deps3 zeros(3,3); zeros(3,3) dR_deps3];

J = -[dM_deta*y_o dM_deps1*y_o dM_deps2*y_o dM_deps3*y_o];