function X_hat = kalman(input)

%Author: Jan Rohde
%Description: A discrete Extended Kalman Filter which returns the estimated
%             states X_hat given 4 inputvariables; B_meas, B_ref, S_meas
%             and S_ref.

global Q H G R q_opt_next q_bar_next omega_bar_next x_bar_next P_bar_next

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Measurements and reference values are loaded
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
B_meas = input(1:3);
B_ref = input(4:6);
S_meas = input(7:9);
S_ref = input(10:12);

% B_meas = [1 2 3]';
% B_ref = [1.1 2.1 3.1]';
% S_meas = [4 5 6]';
% S_ref = [4.1 5.1 6.1]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Gauss-Newton algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
q_opt = q_opt_next;
Y = [q_opt;B_meas;S_meas];
J = genJacobian(Y);
R_opt = rquat(q_opt);
M = [R_opt zeros(3); zeros(3) R_opt];
y_meas = [B_meas; S_meas];
y_ref = [B_ref; S_ref];
q_opt_next = q_opt-inv(J'*J)*J'*(y_ref-M*y_meas);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Update Kalman gain, K. create Kw and Keps
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
P_bar = P_bar_next;
K = P_bar*H'*inv(H*P_bar*H'+R);
Keps = K(1:3,1:3);
Komega = K(4:6,1:3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Updating estimation error. for quaternion part and angular velocity
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
q_bar = q_bar_next;
q_error = qProdinv([q_opt;q_bar]);
deps = q_error(2:4);
delta_eps_w = q_opt(2:4)-q_bar(2:4);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Updating state estimations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%quaternion estimation update
Deps = Keps*deps;
eta = sqrt(1-norm(Deps,2)^2)*sign(q_error(1));
q_est = qProd([eta; Deps; q_bar]);

%omega estimation update
omega_bar = omega_bar_next;
omega_est = omega_bar + Komega*delta_eps_w;

%State estimate updated
x_est = [q_est; omega_est];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Updating error Covariance matrix P
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
P = (eye(6)-K*H)*P_bar*(eye(6)-K*H)'+K*R*K';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Updating state transition matrices, (Forward Euler Integration)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Phi_r = eye(6)+linearsystemR(x_est);    %reduced system
Phi_tot = nonLinearProp(x_est);         %full system

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Propagating state and Error Covariance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%state estimation propagation
x_bar_next = Phi_tot + x_est;
q_bar_next = x_bar_next(1:4);
omega_bar_next = x_bar_next(5:7);
%Normalize quaternion propagation
q_bar_next = q_bar_next/norm(q_bar_next,2);
x_bar_next(1:4) = q_bar_next;
%Error covariance propagation
P_bar_next = Phi_r*P*Phi_r' + G*Q*G';
%Sending estimated states to the output
X_hat = x_est;
