function F = linearsystemR(x_hat)
% Caculates the reduced Linearized System matrix Phi_r at state x_hat
% Author: Bernt Ove Sunde
% input = x_hat = [eta_hat epsilon1_hat epsilon2_hat epsilon3_hat
% omega_(ob,x)^b omega_(ob,y)^b omega_(ob,z)^b]'
% Returns: F = Phi_r

global omega_o;
global I;

w_O = omega_o;

kx = ( I(2,2)-I(3,3) )/I(1,1);
ky = ( I(1,1)-I(3,3) )/I(2,2);
kz = ( I(2,2)-I(1,1) )/I(3,3);

eps = x_hat(2:4);      eta = x_hat(1);        w_B_OB = x_hat(5:7);
eps1 = eps(1);      eps2 = eps(2);      eps3 = eps(3);
w_1 = w_B_OB(1);    w_2 = w_B_OB(2);    w_3 = w_B_OB(3);

F_attitude = [-1/2*smtrx(w_B_OB) 1/2*( eta*eye(3)+smtrx(eps) )];

R_B_O = rquat([eta; eps])';

c1 = R_B_O(:,1);
c2 = R_B_O(:,2);
c3 = R_B_O(:,3);

c12 =R_B_O(1,2);
c13 =R_B_O(1,3);
c22 =R_B_O(2,2);
c23 =R_B_O(2,3);
c32 =R_B_O(3,2);
c33 =R_B_O(3,3);


w = w_B_OB + c2*w_O;

w_x = w(1);
w_y = w(2);
w_z = w(3);


a52 = 2*kx*w_O*( eta*w_y + eps1*w_z ) + 6*kx*w_O^2*( eps1*c23 - eta*c33 ) ...
+ 2*( eta*w_2 - eps1*w_3 )*w_O;
a53 = -2*kx*w_O*( eps3*w_y + eps2*w_z ) + 6*kx*w_O^2*( eps2*c23 - eps3*c33 ) ...
+ 2*( eps2*w_3 - eps3*w_2 )*w_O;
a54 = 2*kx*w_O*( eps3*w_z - eps2*w_y ) - 6*kx*w_O^2*( eps2*c33 + eps3*c23 ) ...
- 2*( eps3*w_3 + eps2*w_2 )*w_O;
a56 = kx*w_z - c32*w_O;
a57 = kx*w_y + c22*w_O;


a62 = 2*ky*w_O*( eps2*w_z - eta*w_x ) + 6*ky*w_O^2*( eps3*c33 - eps1*c13 ) ...
- 2*( eps3*w_1 + eps2*w_3 )*w_O;
a63 = 2*ky*w_O*( eps1*w_z + eps3*w_x ) - 6*ky*w_O^2*( eta*c33 + eps2*c13 ) ...
+ 2*( eps3*w_1 - eps1*w_3 )*w_O;
a64 = 2*ky*w_O*( eta*w_z + eps2*w_x ) + 6*ky*w_O^2*( eps1*c33 + eps3*c13 ) ...
+ 2*( eps2*w_1 - eta*w_3 )*w_O;
a65 = -ky*w_z + c32*w_O;
a67 = -ky*w_x - c12*w_O;


a72 = 2*kz*w_O*( eps2*w_y - eps1*w_x ) + 6*kz*w_O^2*( eps3*c23 + eta*c13 ) ...
+ 2*( eps1*w_1 + eps2*w_2 )*w_O;
a73 = 2*kz*w_O*( eps1*w_y + eps2*w_x ) + 6*kz*w_O^2*( eps3*c13 - eta*c23 ) ...
+ 2*( eps1*w_2 - eps2*w_1 )*w_O;
a74 = 2*kz*w_O*( eta*w_y - eps3*w_x ) + 6*kz*w_O^2*( eps1*c23 + eps2*c13 ) ...
+ 2*( eps3*w_1 + eta*w_2 )*w_O;
a75 = -kz*w_y - c22*w_O;
a76 = -kz*w_x + c12*w_O;


F_velocity = [  a52, a53, a54,  0 , a56, a57;
                a62, a63, a64, a65,  0 , a67;
                a72, a73, a74, a75, a76,  0 ];
F = [ F_attitude;
      F_velocity];